'use strict';

/**
 * Create a new ArmLauncher instance, which swings an arm when it launches.
 * @param {Game} game - The game in which the launcher is placed
 * @param {Number} x - The launcher's starting x-coordinate
 * @param {Number} y - The launcher's starting y-coordinate
 */
function ArmLauncher(game, x, y) {
	this._arm = new Sprite(TextureCache[this.ARM_SPRITE]);
	
	Launcher.call(this, game, x, y);
	
	this._arm.anchor.x = this.ARM_ROTATION_X;
	this._arm.anchor.y = this.ARM_ROTATION_Y;
	this._arm.rotation = this.ARM_START_ROTATION;
	
	game.app.stage.addChild(this._arm);
	game.app.stage.addChild(this._body);
}
// Extend Launcher.
ArmLauncher.prototype = Object.create(Launcher.prototype);

// Define constants.
/** @constant {String} The path from the game root to the sprite for the launcher arm */
ArmLauncher.prototype.ARM_SPRITE = 'images/game/launcher_arm.png';
/** @override @constant {Number} The x-offset of the arm from the launcher body's x-position, in pixels */
ArmLauncher.prototype.ARM_X = 24;
/** @override @constant {Number} The y-offset of the arm from the launcher body's y-position, in pixels */
ArmLauncher.prototype.ARM_Y = 24;
/** @constant {Number} The x-position of the arm's rotation anchor point, from 0 to 1 */
ArmLauncher.prototype.ARM_ROTATION_X = 0.7;
/** @constant {Number} The y-position of the arm's rotation anchor point, from 0 to 1 */
ArmLauncher.prototype.ARM_ROTATION_Y = 0.4;
/** @constant {Number} The starting rotation of the arm, in radians */
ArmLauncher.prototype.ARM_START_ROTATION = -0.03 * Math.TAU;
/** @constant {Number} The end rotation of the arm when it launches, in radians */
ArmLauncher.prototype.ARM_LAUNCH_ROTATION = 0.15 * Math.TAU;

/**
 * Queue assets to be preloaded that might be needed by any Launcher instance.
 */
ArmLauncher.queueAssets = Launcher.queueAssets.bind(ArmLauncher);

/**
 * @override
 * Update the launcher for the next frame.
 * @param {Number} deltaTime - The time since the last frame, in milliseconds
 */
ArmLauncher.prototype.update = function (deltaTime) {
	Launcher.prototype.update.call(this, deltaTime);
	
	if (this._state === this.STATES.LAUNCH) {
		this._arm.rotation = Utils.mappedAbsFunc(this._currentFrame, this.LAUNCH_DURATION, this.ARM_START_ROTATION, this.ARM_LAUNCH_ROTATION);
	}
};
